!classDefinition: #ComposableCalendarTest category: #'ISW1-2019-2C-Parcial'!
TestCase subclass: #ComposableCalendarTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:18:11'!
test01CalendarShouldHaveAllIntervalsAvailableWhenCreatingIt
	
	| available calendar event |
	
	calendar _ ComposableCalendar new.
	event _ SingleEvent on: January/10/2019 between: 10:00 and: 11:00.
	
	available _ calendar isAvailable: event.
	
	self assert: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:20:09'!
test02AddingSingleEventToCalendarShouldMakeThatIntervalUnavailable
	
	| available calendar event |
	
	calendar _ ComposableCalendar new. 
	event _ SingleEvent on: January/10/2019 between: 10:00 and: 11:00.
	
	calendar addEvent: event.
	available _ calendar isAvailable: event.
	
	self deny: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:21:29'!
test03AddingSingleEventToCalendarShouldMakeAnyEventStartingInThatIntervalUnavailable

	| available calendar event unavailableEvent |
	
	calendar _ ComposableCalendar new.
	event _ SingleEvent on: January/10/2019 between: 10:00 and: 11:00.
	unavailableEvent _ SingleEvent on: January/10/2019 between: 10:30 and: 11:30.
	
	calendar addEvent: event.
	available _ calendar isAvailable: unavailableEvent.
	
	self deny: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:22:38'!
test04AddingSingleEventToCalendarShouldMakeAnyEventEndingInThatIntervalUnavailable

	| available calendar event unavailableEvent eventDay |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	event _ SingleEvent on: eventDay between: 10:00 and: 11:00.
	unavailableEvent _ SingleEvent on: eventDay between: 9:30 and: 10:30.
	
	calendar addEvent: event.
	available _ calendar isAvailable: unavailableEvent.
	
	self deny: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:23:30'!
test05AddingSingleEventToCalendarShouldMakeAnyEventSharingTimeWithThatIntervalUnavailable

	| available calendar event unavailableEvent eventDay |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	event _ SingleEvent on: eventDay between: 10:00 and: 11:00.
	unavailableEvent _ SingleEvent on: eventDay between: 9:30 and: 11:30.
	
	calendar addEvent: event.
	available _ calendar isAvailable: unavailableEvent.
	
	self deny: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:24:18'!
test06AddingSingleEventToCalendarShouldMakeAnyEventEndingAtTheStartAvailable

	| available calendar event availableEvent eventDay |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	event _ SingleEvent on: eventDay between: 10:00 and: 11:00.
	availableEvent _ SingleEvent on: eventDay between: 09:00 and: 10:00.
	
	calendar addEvent: event.
	available _ calendar isAvailable: availableEvent.
	
	self assert: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:25:11'!
test07AddingSingleEventToCalendarShouldMakeAnyEventEndingAtTheSameTimeUnavailable

	| available calendar event unavailableEvent endTime eventDay |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	endTime _ 11:00.
	event _ SingleEvent on: eventDay between: 10:00 and: endTime.
	unavailableEvent _ SingleEvent on: eventDay between: 10:30 and: endTime.
	
	calendar addEvent: event.
	available _ calendar isAvailable: unavailableEvent.
	
	self deny: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:32:00'!
test08AddRepeatingEventInOnlyOneDayShouldOccupyCalendarOnThatDay

	| calendar event unavailableEvent eventDay eventDays |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	eventDays _ eventDay to: eventDay.
	event _ RepeatingEvent on: eventDays between: 10:00 and: 11:00. 
	
	unavailableEvent _ SingleEvent on: eventDay between: 10:30 and: 11:30.
	
	calendar addEvent: event.
	
	self deny: (calendar isAvailable: unavailableEvent)
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:33:37'!
test09AddRepeatingEventInOnlyOneDayShouldNotOccupyCalendarOnOtherTimes

	| available calendar event availableEvent eventDay eventDays |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	eventDays _ eventDay to: eventDay.
	event _ RepeatingEvent on: eventDays between: 10:00 and: 11:00. 
	
	availableEvent _ SingleEvent on: eventDay between: 11:00 and: 12:30.
	
	calendar addEvent: event.
	available _ calendar isAvailable: availableEvent.
	
	self assert: available.
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:34:02'!
test10AddRepeatingEventOnManyDaysShouldOccupyCalendarOnAllDays

	| calendar event firstUnavailableEvent eventDay eventDays secondUnavailableEvent |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	eventDays _ eventDay to: eventDay next.
	event _ RepeatingEvent on: eventDays between: 10:00 and: 11:00. 
	
	firstUnavailableEvent _ SingleEvent on: eventDay between: 10:30 and: 11:30.
	secondUnavailableEvent _ SingleEvent on: eventDay next between: 10:30 and: 11:30.
	
	calendar addEvent: event.
	self deny: (calendar isAvailable: firstUnavailableEvent).
	self deny: (calendar isAvailable: secondUnavailableEvent).
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:34:53'!
test11AddInfiniteRepeatingEventShouldNotHangWhenAskingForAnAvailableEvent

	| calendar event availableEvent eventDay eventDays |
	
	calendar _ ComposableCalendar new.
	eventDay _ January/10/2019.
	eventDays _ eventDay to: FixedGregorianDate theEndOfTime.
	event _ RepeatingEvent on: eventDays between: 10:00 and: 11:00. 
	
	availableEvent _ SingleEvent on: eventDay previous between: 10:30 and: 11:30.
	
	calendar addEvent: event.
	self assert: (calendar isAvailable: availableEvent).
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:45:25'!
test12AddingEventToChildCalendarShouldOccupyTimeInParentCalendar

	| childCalendar event parentCalendar |
	
	parentCalendar _ ComposableCalendar new.
	childCalendar _ ComposableCalendar new.
	
	event _ SingleEvent on: January/10/2019 between: 10:00 and: 11:00.
	childCalendar addEvent: event.
	
	parentCalendar addCalendar: childCalendar.
	
	self deny: (parentCalendar isAvailable: event).
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 19:49:08'!
test13AddingEventToParentCalendarAddsItToChildCalendar

	| childCalendar event parentCalendar |
	
	parentCalendar _ ComposableCalendar new.
	childCalendar _ ComposableCalendar new.
	
	parentCalendar addCalendar: childCalendar.
	
	event _ SingleEvent on: January/10/2019 between: 10:00 and: 11:00.
	parentCalendar addEvent: event.
	
	self deny: (childCalendar isAvailable: event).
	! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 20:25:53'!
test14FindEventForOneDayInEmptyCalendarShouldReturnEventInThatTime

	| calendar date daysInterval duration found |
	
	calendar _ ComposableCalendar new.
	
	duration _ 1 * hour.
	date _ January/10/2019.
	daysInterval _ date to: date.
	found _ calendar findEventFor: duration on: daysInterval between: 10:00 and: 11:00.
	
	self assert: date equals: found date.
	self assert: 10:00 equals: found startTime.
	self assert: 11:00 equals: found endTime.! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 20:36:17'!
test15FindEventForAnOccupiedDayShouldRaiseException

	| calendar date daysInterval duration |
	
	calendar _ ComposableCalendar new.
	
	duration _ 1 * hour.
	date _ January/10/2019.
	daysInterval _ date to: date.
	calendar addEvent: (SingleEvent on: date between: 10:00 and: 11:00).
	self should: [ calendar findEventFor: duration on: daysInterval between: 10:00 and: 10:00 ]
		raise: Error
		withExceptionDo: [ :exception |
			self 
				assert: ComposableCalendar couldntFindTimeForEventErrorMessage 
				equals: exception messageText ].! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 20:38:17'!
test16FindEventWhenFirstHourIsntAvailableShouldSearchInConsequentHours

	| calendar date daysInterval duration found |
	
	calendar _ ComposableCalendar new.
	
	duration _ 1 * hour.
	date _ January/10/2019.
	daysInterval _ date to: date.
	calendar addEvent: (SingleEvent on: date between: 10:00 and: 11:00).
	
	found _ calendar findEventFor: duration on: daysInterval between: 10:00 and: 11:00.
	
	self assert: date equals: found date.
	self assert: 11:00 equals: found startTime.
	self assert: 12:00 equals: found endTime.! !

!ComposableCalendarTest methodsFor: 'tests' stamp: 'EB 7/4/2019 20:49:35'!
test17FindEventWhenFirstDateIsntAvailableShouldSearchInConsequentDates

	| calendar date daysInterval duration found |
	
	calendar _ ComposableCalendar new.
	
	duration _ 1 * hour.
	date _ January/10/2019.
	daysInterval _ date to: date next.
	calendar addEvent: (SingleEvent on: date between: 10:00 and: 12:00).
	
	found _ calendar findEventFor: duration on: daysInterval between: 10:00 and: 11:00.
	
	self assert: date next equals: found date.
	self assert: 10:00 equals: found startTime.
	self assert: 11:00 equals: found endTime.! !


!classDefinition: #SingleEventTest category: #'ISW1-2019-2C-Parcial'!
TestCase subclass: #SingleEventTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!SingleEventTest methodsFor: 'as yet unclassified' stamp: 'EB 7/4/2019 21:53:14'!
test01CantCreateAnEventWithEndBeforeStart
	
	self should: [ SingleEvent on: January/10/2019 between: 10:00 and: 9:00 ]
		raise: Error
		withExceptionDo: [ :exception |
			self assert: Event endTimeCannotBeBeforeStartTimeErrorMessage equals: exception messageText ].! !


!classDefinition: #Calendar category: #'ISW1-2019-2C-Parcial'!
Object subclass: #Calendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!Calendar methodsFor: 'adding events' stamp: 'EB 7/4/2019 20:18:16'!
addEvent: anEvent
	self subclassResponsibility.! !


!Calendar methodsFor: 'checking availability' stamp: 'EB 7/4/2019 20:18:19'!
isAvailable: aSingleEvent
	self subclassResponsibility.! !


!classDefinition: #CalendarNode category: #'ISW1-2019-2C-Parcial'!
Calendar subclass: #CalendarNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!CalendarNode methodsFor: 'adding calendars' stamp: 'EB 7/4/2019 20:00:22'!
withCalendar: aCalendar
	self subclassResponsibility.! !


!classDefinition: #CalendarContainer category: #'ISW1-2019-2C-Parcial'!
CalendarNode subclass: #CalendarContainer
	instanceVariableNames: 'calendars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!CalendarContainer methodsFor: 'initializing' stamp: 'EB 7/4/2019 20:12:40'!
initializeWithChildren: someCalendarChildren 
	calendars _ someCalendarChildren.! !


!CalendarContainer methodsFor: 'adding calendars' stamp: 'EB 7/4/2019 20:01:35'!
withCalendar: aCalendar
	calendars add: aCalendar.! !


!CalendarContainer methodsFor: 'checking availability' stamp: 'EB 7/4/2019 20:09:00'!
isAvailable: aSingleEvent
	^calendars allSatisfy: [ :calendar | calendar isAvailable: aSingleEvent ].! !


!CalendarContainer methodsFor: 'adding events' stamp: 'EB 7/4/2019 20:07:04'!
addEvent: anEvent
	calendars do: [ :calendar | calendar addEvent: anEvent ].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'CalendarContainer class' category: #'ISW1-2019-2C-Parcial'!
CalendarContainer class
	instanceVariableNames: ''!

!CalendarContainer class methodsFor: 'instance creation' stamp: 'EB 7/4/2019 20:12:15'!
withChildren: someCalendarChildren
	^self new initializeWithChildren: someCalendarChildren.! !


!classDefinition: #SingleCalendar category: #'ISW1-2019-2C-Parcial'!
CalendarNode subclass: #SingleCalendar
	instanceVariableNames: 'events'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!SingleCalendar methodsFor: 'adding events' stamp: 'EB 7/4/2019 20:06:37'!
addEvent: anEvent
	events add: anEvent.! !


!SingleCalendar methodsFor: 'initializing' stamp: 'EB 7/4/2019 20:11:12'!
initialize
	events _ Set new.! !


!SingleCalendar methodsFor: 'checking availability' stamp: 'EB 7/4/2019 20:08:28'!
isAvailable: aSingleEvent
	^events noneSatisfy: [ :event | event sharesTimeWith: aSingleEvent ].
! !


!SingleCalendar methodsFor: 'adding calendars' stamp: 'EB 7/4/2019 20:14:50'!
withCalendar: aCalendar
	^CalendarContainer withChildren: (OrderedCollection with: self with: aCalendar).
! !


!classDefinition: #ComposableCalendar category: #'ISW1-2019-2C-Parcial'!
Calendar subclass: #ComposableCalendar
	instanceVariableNames: 'calendarNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!ComposableCalendar methodsFor: 'adding events' stamp: 'EB 7/4/2019 20:10:27'!
addEvent: anEvent
	calendarNode addEvent: anEvent.! !


!ComposableCalendar methodsFor: 'checking availability' stamp: 'EB 7/4/2019 20:09:48'!
isAvailable: aSingleEvent
	^calendarNode isAvailable: aSingleEvent.! !


!ComposableCalendar methodsFor: 'initializing' stamp: 'EB 7/4/2019 20:15:38'!
initialize
	calendarNode _ SingleCalendar new.! !


!ComposableCalendar methodsFor: 'adding calendars' stamp: 'EB 7/4/2019 20:04:20'!
addCalendar: aCalendar
	calendarNode _ calendarNode withCalendar: aCalendar.! !


!ComposableCalendar methodsFor: 'event finding' stamp: 'EB 7/4/2019 20:54:17'!
findEventFor: anEventDuration on: aDatesInterval between: aStartTime and: anEndTime
	
	| event |
	
	aDatesInterval 
		detect: [ :possibleDate |
			(aStartTime to: anEndTime by: 1 * hour) 
				detect: [ :possibleStartTime |
					event _ SingleEvent on: possibleDate startingAt: possibleStartTime withDuration: anEventDuration.
					calendarNode isAvailable: event ]
				ifFound: [ :startTime | true ] 
				ifNone: [ false ].
		]
		ifNone: [
			self error: self class couldntFindTimeForEventErrorMessage].
	
	^event.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'ComposableCalendar class' category: #'ISW1-2019-2C-Parcial'!
ComposableCalendar class
	instanceVariableNames: ''!

!ComposableCalendar class methodsFor: 'error messages' stamp: 'EB 7/4/2019 20:33:32'!
couldntFindTimeForEventErrorMessage
	^'There is no time available for an event in the intervals specified.'.! !


!classDefinition: #Event category: #'ISW1-2019-2C-Parcial'!
Object subclass: #Event
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!


!Event methodsFor: 'evaluating' stamp: 'EB 7/4/2019 19:35:49'!
sharesTimeWith: aSingleEvent
	self subclassResponsibility.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'Event class' category: #'ISW1-2019-2C-Parcial'!
Event class
	instanceVariableNames: ''!

!Event class methodsFor: 'error messages' stamp: 'EB 7/4/2019 21:54:33'!
endTimeCannotBeBeforeStartTimeErrorMessage
	^'End time cannot be before start time.'.! !


!classDefinition: #RepeatingEvent category: #'ISW1-2019-2C-Parcial'!
Event subclass: #RepeatingEvent
	instanceVariableNames: 'dates startTime endTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!



!RepeatingEvent methodsFor: 'evaluating' stamp: 'EB 7/4/2019 19:30:37'!
sharesTimeWith: aSingleEvent
	
	| eventDay |
	
	eventDay _ aSingleEvent date.
	^(dates includes: eventDay) and: [
		(self eventAtDay: eventDay) sharesTimeWith: aSingleEvent ].! !


!RepeatingEvent methodsFor: 'initializing' stamp: 'EB 7/4/2019 19:31:05'!
initializeOn: aDatesInterval between: aStartTime and: anEndTime 
	dates _ aDatesInterval.
	startTime _ aStartTime.
	endTime _ anEndTime.! !


!RepeatingEvent methodsFor: 'generating events' stamp: 'EB 7/4/2019 19:33:12'!
eventAtDay: aDay
	^SingleEvent on: aDay between: startTime and: endTime.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'RepeatingEvent class' category: #'ISW1-2019-2C-Parcial'!
RepeatingEvent class
	instanceVariableNames: ''!


!RepeatingEvent class methodsFor: 'instance creation' stamp: 'EB 7/4/2019 19:29:52'!
on: aDaysInterval between: aStartTime and: anEndTime 
	^self new initializeOn: aDaysInterval between: aStartTime and: anEndTime.! !


!classDefinition: #SingleEvent category: #'ISW1-2019-2C-Parcial'!
Event subclass: #SingleEvent
	instanceVariableNames: 'date startTime endTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ISW1-2019-2C-Parcial'!

!SingleEvent methodsFor: 'initializing' stamp: 'EB 7/4/2019 19:19:49'!
initializeOn: aDate between: aStartTime and: anEndTime 
	date _ aDate.
	startTime _ aStartTime.
	endTime _ anEndTime.! !


!SingleEvent methodsFor: 'evaluating' stamp: 'EB 7/4/2019 19:20:35'!
sharesTimeWith: aSingleEvent
	
	| eventEnd eventStart otherEventEnd otherEventStart |
	
	^self date = aSingleEvent date and: [
		eventStart _ self startTime. 
		eventEnd _ self endTime.
		otherEventStart _ aSingleEvent startTime.
		otherEventEnd _ aSingleEvent endTime.
		otherEventStart < eventEnd and: otherEventEnd > eventStart ].! !


!SingleEvent methodsFor: 'accessing' stamp: 'EB 7/4/2019 19:16:54'!
date
	^date.! !

!SingleEvent methodsFor: 'accessing' stamp: 'EB 7/4/2019 19:20:27'!
endTime
	^endTime.! !

!SingleEvent methodsFor: 'accessing' stamp: 'EB 7/4/2019 19:20:39'!
startTime
	^startTime.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'SingleEvent class' category: #'ISW1-2019-2C-Parcial'!
SingleEvent class
	instanceVariableNames: ''!

!SingleEvent class methodsFor: 'instance creation' stamp: 'EB 7/4/2019 21:54:54'!
on: aDate between: aStartTime and: anEndTime
	anEndTime < aStartTime ifTrue: [
		self error: Event endTimeCannotBeBeforeStartTimeErrorMessage ].
	^self new initializeOn: aDate between: aStartTime and: anEndTime.! !

!SingleEvent class methodsFor: 'instance creation' stamp: 'EB 7/4/2019 20:53:43'!
on: aDate startingAt: aStartTime withDuration: aDuration
	
	| endTime |
	
	endTime _ aStartTime next: aDuration.
	^SingleEvent on: aDate between: aStartTime and: endTime.
						! !
